import { Action } from "tandem-common/actions";
import { IActor } from "tandem-common/actors";
import { toArray } from "tandem-common/utils/array";
import { IRange, IPoint } from "tandem-common/geom";
import { IEditor, IEditorTool, IHistoryItem } from "tandem-front-end/models";
import { EditorToolFactoryDependency } from "tandem-front-end/dependencies";

export class MouseAction extends Action {

  static readonly CANVAS_MOUSE_DOWN = "canvasMouseDown";
  static readonly SELECTION_DOUBLE_CLICK = "selectionDoubleClick";

  constructor(type, readonly originalEvent: MouseEvent) {
    super(type);
    Object.assign(this, {
      clientX : originalEvent.clientX,
      clientY : originalEvent.clientY,
      metaKey : originalEvent.metaKey
    });
  }
  preventDefault() {
    this.originalEvent.preventDefault();
  }
}

export class KeyboardAction extends Action {

  static readonly CANVAS_KEY_DOWN = "canvasKeyDown";

  readonly keyCode: number;
  readonly which: number;
  constructor(type, readonly originalEvent: KeyboardEvent) {
    super(type);
    Object.assign(this, {
      which : originalEvent.which,
      keyCode: originalEvent.keyCode
    });
  }

  preventDefault() {
    this.originalEvent.preventDefault();
  }
}

export class SelectAction extends Action {

  static readonly SELECT = "SELECT";

  public items: Array<any>;
  public keepPreviousSelection: boolean;
  public toggle: boolean;

  constructor(items: any = undefined, keepPreviousSelection = false, toggle = false) {
    super(SelectAction.SELECT);
    this.items = toArray(items);
    this.keepPreviousSelection = !!keepPreviousSelection;
    this.toggle = toggle;
  }
}


export class SelectAllAction extends Action {
  static readonly SELECT_ALL = "selectAll";
  constructor() {
    super(SelectAllAction.SELECT_ALL);
  }
}

export class SelectSourceAtOffsetAction extends Action {
  static readonly SELECT_SOURCE_AT_OFFSET = "selectAtSourceOffset";
  readonly data: Array<IRange>;
  constructor(...data: Array<IRange>) {
    super(SelectSourceAtOffsetAction.SELECT_SOURCE_AT_OFFSET);
    this.data = data;
  }
}

export class ToggleSelectAction extends SelectAction {
  constructor(items = undefined, keepPreviousSelection: boolean = false) {
    super(items, keepPreviousSelection, true);
  }
}

export class ZoomAction extends Action {
  static readonly ZOOM = "zoom";
  constructor(readonly delta: number, readonly ease: boolean = false) {
    super(ZoomAction.ZOOM);
  }
}

export class PasteAction extends Action {
  static readonly PASTE = "paste";
  constructor(readonly item: DataTransferItem) {
    super(PasteAction.PASTE);
  }
}

export class DeleteSelectionAction extends Action {
  static readonly DELETE_SELECTION = "deleteSelection";
  constructor() {
    super(DeleteSelectionAction.DELETE_SELECTION);
  }
}

export class SetToolAction extends Action {
  static readonly SET_TOOL = "setTool";
  constructor(readonly toolFactory: { create(editor: IEditor): IEditorTool }) {
    super(SetToolAction.SET_TOOL);
  }
}

export class KeyCommandAction extends Action {
  static readonly KEY_COMMAND = "keyCommand";
  constructor(readonly combo: string) {
    super(KeyCommandAction.KEY_COMMAND);
  }
}

export class UndoAction extends Action {
  static readonly UNDO = "undo";
  constructor() {
    super(UndoAction.UNDO);
  }
}

export class RedoAction extends Action {
  static readonly REDO = "redo";
  constructor() {
    super(RedoAction.REDO);
  }
}

export class RemoveSelectionAction extends Action {
  static readonly REMOVE_SELECTION = "removeSelection";
  constructor() {
    super(RemoveSelectionAction.REMOVE_SELECTION);
  }
}
